#!/usr/local/bin/perl -wI.

package TWiki::Contrib::IndexGen;

use strict;

use CGI;
use TWiki::Func;
use TWiki::UI::View;
use File::Temp qw/ tempfile /;
use Error qw( :try );

use List::Compare::Functional qw( get_unique );

use vars qw( $VERSION $RELEASE );

# This should always be $Rev: 9716 $ so that TWiki can determine the checked-in
# status of the plugin. It is used by the build automation tools, so
# you should leave it alone.
$VERSION = '$Rev: 9716 $';

# This is a free-form string you can use to "name" your own plugin version.
# It is *not* used by the build automation tools, but is reported as part
# of the version number in PLUGINDESCRIPTIONS.
$RELEASE = 'Dakar';


=pod

=head1 TWiki::Contrib::IndexGen

TWiki::Contrib::IndexGen - generates Index for you

=head1 DESCRIPTION

Some description

=cut


=pod

=head2 saveArrange($webName, $userName, $thePathInfo, $query)

saves the changes.

=cut

sub saveArrange {
  my( $webName, $userName, $thePathInfo, $query) = @_;

  if(TWiki::Func::webExists($webName)) {

    # unlock the page
    TWiki::Func::setTopicEditLock( $webName, "_index", 0 );
    
    my $topicName   = "_index";

    my $session     = new TWiki($userName, $query);
    my $URLTemplate = TWiki::Func::getPreferencesValue( "INDEXGENURL", $webName ) || "";

    # create _index page if it does not exists
    if(!TWiki::Func::topicExists($webName, $topicName)) {
      my $newMeta = new TWiki::Meta($session, $webName, $topicName);
      my $ret     = TWiki::Func::saveTopic( $webName, $topicName, $newMeta, "_some text_", {} );
    }

    my $maxn = $query->param("pagesn") || 0;

    # create _index page

    my $tmpstr  = "";
    my $htmlstr = "";
    my $allstr  = "";

    $htmlstr .= "<ul>\n";

    for(my $i=0; $i < $maxn; $i++) {
      if($query->param("type_$i") eq "0") {
	$htmlstr .= '<li class="heading">'.$query->param("value_$i").'</li>';
      } elsif($query->param("type_$i") eq "2") {
	$htmlstr .= '<li class="title">'.$query->param("value_$i").'</li>';
      } else {
	if($URLTemplate eq "") {
	  $htmlstr .= '<li>[['.$query->param("page_$i").']['.$query->param("value_$i").']]</li>';
	} else {
	  $htmlstr .= '<li><a href="'.TWiki::Func::expandCommonVariables($URLTemplate, $query->param("page_$i"), $webName).'">'.$query->param("value_$i").'</a></li>';
	}
	$allstr .= "%INCLUDE{".$query->param("page_$i")."}%";
      }

      $tmpstr .= $query->param("type_$i")."\n";
      $tmpstr .= $query->param("page_$i")."\n";
      $tmpstr .= $query->param("value_$i")."\n";
    }

    $htmlstr .= "</ul>\n";

    my $outputFile = new File::Temp(TEMPLATE => 'IndexGenXXXXXXXXXX',
				    DIR => File::Spec->tmpdir(),
   			            UNLINK => 0, 
				    SUFFIX => '.txt');

    print $outputFile $tmpstr;

    TWiki::Func::saveAttachment($webName, $topicName, 'TOC.txt',
				      { file => $outputFile,
					comment => 'Table of contents',
					hide => 0,
					dontlog => 1} );

    #close($outputFile);
    ## save the new text
    TWiki::Func::saveTopicText($webName,$topicName, $htmlstr);

    # create _all page ...
    TWiki::Func::saveTopicText($webName, "_all", $allstr);

    $query->delete_all();

    # redirect us to the same page
    TWiki::Func::redirectCgiQuery($query, "");
  }
}


=pod

=head2 viewArrange($webName, $userName, $thePathInfo, $query)

Show the edit form.

=cut

sub viewArrange {
  my( $webName, $userName, $thePathInfo, $query) = @_;

  print $query->header(-type=>'text/html', -expire=>'now');

  $thePathInfo =~ /^\/(.+)(\/){0,1}/;
  $webName = $1;

  if(!TWiki::Func::checkAccessPermission("CHANGE", $userName, undef, "_index", $webName)) {
    die "not allowed";
  }

  my $templateName = TWiki::Func::getPreferencesValue("INDEXGENTEMPLATE", $webName) || "indexgen.tmpl";
  my $tmpl = TWiki::Func::readTemplate($templateName);

  my $htmlCode =<<END;
   <script type="text/javascript" src="%PUBURLPATH%/TWiki/MochiKit/MochiKit.js"></script>
   <script type="text/javascript" src="%PUBURLPATH%/TWiki/MochiKit/New.js"></script>
   <script type="text/javascript" src="%PUBURLPATH%/TWiki/MochiKit/Visual.js"></script>
   <script type="text/javascript" src="%PUBURLPATH%/TWiki/MochiKit/Signal.js"></script>
   <script type="text/javascript" src="%PUBURLPATH%/TWiki/MochiKit/DragAndDrop.js"></script>
   <script type="text/javascript" src="%PUBURLPATH%/TWiki/MochiKit/Sortable.js"></script>

  <script>
// <![CDATA[
    if(typeof(IndexGen) == 'undefined') {
      IndexGen = {}
    }

    // IndexGen.Base

    if(typeof(IndexGen.Base) == 'undefined') {
     IndexGen.Base = {}
    }

    IndexGen.Base.web_topics    = new Array();
    IndexGen.Base.edited_topics = new Array();
    IndexGen.Base.edited_topics_n = 0;

    MochiKit.Base.update(IndexGen.Base, {
      addTopic: function(topicType, eid, topicName, topicValue, originalValue) {
        var item = new Array();

        item.push(eid, topicType, topicName, topicValue, originalValue);

        IndexGen.Base.edited_topics.push(item);
      },

      getTopicWithID: function(eid) {
       for(var i = 0; i < IndexGen.Base.edited_topics.length; i++) {
           if(IndexGen.Base.edited_topics[i][0] == eid)
             return IndexGen.Base.edited_topics[i];
       }

       return null;
      },

      hasTopic: function(topicName) {
       for(var i = 0; i<IndexGen.Base.edited_topics.length; i++) {
           if(IndexGen.Base.edited_topics[i][2] == topicName)
             return true;
       }

       return false;
      },


    addTopic2: function() {
      var l   = \$("thelist2");
      var eid = "thelist1_"+(IndexGen.Base.edited_topics.length+1);

      IndexGen.Base.addTopic(0, eid, "---", "Click here to edit chapter title", "Click here to edit chapter title");

      // make Separator link
      l.appendChild(LI({"class": "heder", "id": eid}, IndexGen.Editor.makeLink(eid, "Click here to edit chapter title")));

      self = MochiKit.Sortable.Sortable;
      options = self.sortables["thelist1"]
        var options_for_draggable = {
            revert: true,
            ghosting: options.ghosting,
            scroll: options.scroll,
            scrollSensitivity: options.scrollSensitivity,
            scrollSpeed: options.scrollSpeed,
            constraint: options.constraint,
            handle: options.handle
        };

        var options_for_droppable = {
            overlap: options.overlap,
            containment: options.containment,
            hoverclass: options.hoverclass,
            onhover: self.onHover,
            tree: options.tree,
            accept: options.accept };

      MochiKit.Base.map(function (e) {
            var handle = options.handle ?
                MochiKit.DOM.getFirstElementByTagAndClassName(null,
                    options.handle, e) : e;
            options.draggables.push(
                new MochiKit.DragAndDrop.Draggable(e,
                    MochiKit.Base.update(options_for_draggable,
                                         {handle: handle})));
            new MochiKit.DragAndDrop.Droppable(e, options_for_droppable);
            if (options.tree) {
                e.treeNode = element;
            }
            options.droppables.push(e);
        }, [\$(eid)]);

    },

    doExport: function() {

      if(confirm("Do you want to save the changes?")) {
        if(IndexGen.Editor.selected_item != null)  {
            IndexGen.Editor.closeEditor(\$("embededitor"), IndexGen.Editor.selected_item);
        }

        var s = MochiKit.Sortable.Sortable.serialize('thelist2');
        var element = s.split("&");
        var eid_elements = new Array();

        for(var i = 0; i < element.length; i++) {
            var idx = element[i].lastIndexOf("=");
            var eid = element[i].substr(idx+1);
            eid_elements.push(eid);
        }

        var tmp = "";
        var postform = \$("postform");
        var n = 0;

        MochiKit.Base.map(function (eid) {
            var elem = IndexGen.Base.getTopicWithID("thelist1_"+eid);

            appendChildNodes(postform, INPUT({"type": "hidden", "name":"page_"+n, "value": elem[2]}));
            appendChildNodes(postform, INPUT({"type": "hidden", "name":"value_"+n, "value": elem[3]}));
            appendChildNodes(postform, INPUT({"type": "hidden", "name":"type_"+n, "value": elem[1]}));

            n += 1;
        }, eid_elements);

        postform.pagesn.value = n;
        postform.submit();
     }
    }
    });

    // IndexGen.Editor 

    if(typeof(IndexGen.Editor) == 'undefined') {
     IndexGen.Editor = {}
    }

    IndexGen.Editor.selected_item = null;

    MochiKit.Base.update(IndexGen.Editor, {
      closeEditor: function(myform, eid) {
          var elem = IndexGen.Base.getTopicWithID(eid);
          var text = myform.elements[0].value;

          elem[3] = text;
          replaceChildNodes(\$(eid), IndexGen.Editor.makeLink(eid, text));

          IndexGen.Editor.selected_item = null;
      },

      cancelEditor: function(eid) {
          var elem = IndexGen.Base.getTopicWithID(eid);
          replaceChildNodes(\$(eid), IndexGen.Editor.makeLink(eid, elem[3]));

          IndexGen.Editor.selected_item = null;
      },

      undoEditor: function(eid) {
          var form = \$("embededitor");
          var elem = IndexGen.Base.getTopicWithID(eid);

          form.title.value = elem[4];
      },


      makeEditor: function(eid) {
          if(IndexGen.Editor.selected_item != null)  {
              IndexGen.Editor.closeEditor(\$("embededitor"), IndexGen.Editor.selected_item);
          }

//          if(!IndexGen.Editor.selected_item) {
          if(1) {

              var elem  = IndexGen.Base.getTopicWithID(eid);
              var value = elem[3];

              var myform = FORM({"style": "margin: 0px",
                                 "id": "embededitor",
                                 "method": "GET", 
                                 "action": "javascript:void(0)",
                                 "onsubmit": "IndexGen.Editor.closeEditor(this, '"+eid+"')"},
                                 INPUT({"name": "title", "type": "text", "value": value}),
                                    A({"href":"javascript:void(0)", "onclick": "IndexGen.Editor.undoEditor('"+eid+"')"}, "undo"),
                                    " | ",
                                    A({"href":"javascript:void(0)", "onclick": "IndexGen.Editor.cancelEditor('"+eid+"')"}, "cancel"));

              replaceChildNodes(\$(eid), myform);

              myform.title.focus();
              myform.title.select();

              IndexGen.Editor.selected_item = eid;
          }
      },

      makeLink: function(eid, name) {
          var _class = "topic";

          var elem = IndexGen.Base.getTopicWithID(eid);
          if(elem[1] == 0)
             _class ="heder";

          return DIV({"ondblclick": "IndexGen.Editor.makeEditor('"+eid+"')"}, DIV({"style": "float: right"},
                        A({"href": "javascript:void(0)",
                           "onclick": "IndexGen.Editor.makeEditor('"+eid+"')"}, "edit")),
                        DIV(null, A({"href": "javascript:void(0)",
                                     "class": _class,
                                     "ondblclick": "IndexGen.Editor.makeEditor('"+eid+"')"},
                                     name)));
      },

      makeElement: function(eid, name) {
          return LI({"id": eid}, IndexGen.Editor.makeLink(eid, name));
      }
    });

    function init() {
       var n = 1;
       var thelist2   = \$("thelist2");
       var eid        = "";

END

if(TWiki::Func::webExists($webName)) {
  my @excludeList = split /,/, TWiki::Func::getPreferencesValue( "INDEXGENIGNORE", $webName );
  my @topicList   = TWiki::Func::getTopicList($webName);
  my $isinside    = 0;
  my $hasData     = 0;

  foreach my $topicName (get_unique({lists => [\@topicList, \@excludeList]})) {
       $htmlCode .= '          IndexGen.Base.web_topics.push("'.$topicName.'");'."\n"
  }

  if(TWiki::Func::topicExists($webName, "_index")) {
    
    if(TWiki::Func::attachmentExists($webName, "_index", "TOC.txt")) {
      try {
	my $attdata = TWiki::Func::readAttachment($webName, "_index", "TOC.txt");
	my @lines = split /\n/, $attdata;

	$hasData = 1;

	for(my $i = 0; $i < (@lines/3); $i++) {
	  $htmlCode .= 'eid = "thelist1_"+n; ';
          $htmlCode .= 'IndexGen.Base.addTopic('.$lines[$i*3].', eid, "'.$lines[$i*3+1].'", "'.$lines[$i*3+2].'", "'.$lines[$i*3+2].'");';
	  if($lines[$i*3] == 1) {
	    $htmlCode .= 'appendChildNodes(thelist2, IndexGen.Editor.makeElement(eid, "'.$lines[$i*3+2].'"));';
	  } elsif($lines[$i*3] == 2) {
	    $htmlCode .= 'appendChildNodes(thelist2, LI({"class": "title", "id": eid}, IndexGen.Editor.makeLink(eid, "'.$lines[$i*3+2].'")));';
          } else {
	    $htmlCode .= 'appendChildNodes(thelist2, LI({"class": "heder", "id": eid}, IndexGen.Editor.makeLink(eid, "'.$lines[$i*3+2].'")));';

	  }
	  $htmlCode .= "n+= 1;\n";
	}

      } catch TWiki::AccessControlException with {

      };
    } 
  } 

  if(!$hasData) {
    $htmlCode .= 'eid = "thelist1_"+n; '."\n";
    $htmlCode .= 'IndexGen.Base.addTopic(2, eid, "TITLE", "TITLE", "TITLE");'."\n";
    $htmlCode .= 'appendChildNodes(thelist2, LI({"class": "title", "id": eid}, IndexGen.Editor.makeLink(eid, "TITLE")));'."\n";
    $htmlCode .= "n += 1;";
  }
}


$htmlCode .= <<END;
       MochiKit.Base.map(function (e) {
         var a   = \$("thelist1");
         var eid = "thelist1_"+n;

         if(!IndexGen.Base.hasTopic(e)) {
           IndexGen.Base.addTopic(1, eid, e, e, e);
           appendChildNodes(a, IndexGen.Editor.makeElement(eid, e));
           n+=1;
         }
       }, IndexGen.Base.web_topics);

       MochiKit.Sortable.Sortable.create('thelist1',{containment:['thelist1','thelist2'], dropOnEmpty:true, constraint: false});
       MochiKit.Sortable.Sortable.create('thelist2',{containment:['thelist1','thelist2'], dropOnEmpty:true, constraint: false});
    }
// ]]>

  </script>

  <style type="text/css" media="screen">
    A {
       text-decoration: None;
    }

    ul { 
       border: 1px solid black;
       list-style-type: none;
       padding-left: 0px;
    }

    li {
      border: 1px solid black;
      padding-left: 10px;
      padding-right: 10px;
    }

    .heder { color: red; background-color: yellow}

    .topic { color: black; }

    .title { color: black; background-color: #f0f0f0}
    
  </style>
END

  my ($oopsUrl, $loginName, $unlockTime) = TWiki::Func::checkTopicEditLock($webName, "_index");

  my $htmlCode2 = "";

  if($loginName ne "") {
    $htmlCode2 .= "<h3>User $loginName is maybe editing this page!</h3>";
  } else {
    TWiki::Func::setTopicEditLock( $webName, "_index", 1 );
  }

  $htmlCode2 .=<<END;
<!-- start of table -->

<table border="0">
 <tr>
  <td valign="top">
   <div style="width: 300px">
    <ul id="thelist1">
    </ul>
   </div>
  </td>

  <td align="center" valign="center" width="100">
    <form name="postform" id="postform" method="POST" action="">
     <input type="hidden" name="pagesn" value=""/>
    </form>

    <form method="GET" action="javascript:void(0)" onsubmit="javascript: return false">
      <input type="button" name="left" value="<-" disabled="true"/>
      <br/>
      <input type="button" name="left" value="---" onclick="IndexGen.Base.addTopic2(); return false;">
      <br/>
      <input type="button" name="left" value="->" disabled="true"/>
      <br/>
      <br/>
      <br/>
      <br/>
      <br/>
      <br/>
      <input type="button" name="left" value="save" onclick="IndexGen.Base.doExport(); return false;">
    </form>
  </td>

  <td valign="top">
   <div style="width: 300px">
     <ul id="thelist2">
<!--       <li id="thelist2_10000">&nbsp;</li> -->
     </ul>

     <div style="font-size:8pt; color: #a0a0a0">Drag and drop title here. Double click to edit.</div>

   </div>
  </td>
 </tr>
</table>

<!-- end of table -->
END

  $tmpl =~ s/\%HEAD_INDEXGEN\%/$htmlCode/g;
  $tmpl =~ s/\%BODY_INDEXGEN\%/$htmlCode2/g;

  $tmpl    = TWiki::Func::expandCommonVariables($tmpl, "_index", "TWiki");

  print $tmpl;
}

1;

